/*
 * Copyright (C) 2012 Hermann Meyer, Andreas Degert, Pete Shorthose, Steve Poskitt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 * --------------------------------------------------------------------------
 */

////////////////////////////// LOCAL INCLUDES //////////////////////////

#include "gx_common.h"      // faust support and denormal protection (SSE)
#include "gxtubedelay.h"        // define struct PortIndex
#include "gx_pluginlv2.h"   // define struct PluginLV2
#include "gxtubedelay.cc"    // dsp class generated by faust -> dsp2cc
#ifndef __SSE__
#include "noiser.cc"
#endif


////////////////////////////// MONO ////////////////////////////////////

class GxTubeDelay
{
private:
  // internal stuff
  float*                       output;
  float*                       input;
  PluginLV2*			tubedelay;
#ifndef __SSE__
  PluginLV2*                   wn;
#endif
public:

  void clean();
  inline void run_dsp_mono(uint32_t n_samples);
  void connect_mono(uint32_t port,void* data);
  inline void init_dsp_mono(uint32_t rate);
  inline void connect_all_mono_ports(uint32_t port, void* data);
  void activate_f();
   // constructor
  GxTubeDelay() :
    output(NULL),
    input(NULL),  
    tubedelay(gxtubedelay::plugin()) {};
  // destructor
  ~GxTubeDelay()
  {
  };
};

// plugin stuff

void GxTubeDelay::init_dsp_mono(uint32_t rate)
{
  AVOIDDENORMALS();
#ifndef __SSE__
  wn = noiser::plugin();
  wn->set_samplerate(rate, wn);
#endif
  tubedelay->set_samplerate(static_cast<unsigned int>(rate), tubedelay );
}

void GxTubeDelay::connect_mono(uint32_t port,void* data)
{
 //  std::cout << "connect_mpno " << std::endl;
 //std::cout << "port = " << port << std::endl;
  switch ((PortIndex)port)
    {
    case EFFECTS_OUTPUT:
      output = static_cast<float*>(data);
      break;
    case EFFECTS_INPUT:
      input = static_cast<float*>(data);
      break;
    default:
      break;
    }
}

void GxTubeDelay::activate_f()
{
//   std::cout << "activate_f" << std::endl;
    tubedelay->clear_state(tubedelay);
}

void GxTubeDelay::run_dsp_mono(uint32_t n_samples)
{
  // run dsp
  //std::cout << "run_dsp"<< std::endl;
#ifndef __SSE__
  wn->mono_audio(static_cast<int>(n_samples), input, input, wn);;
#endif
  tubedelay->mono_audio(static_cast<int>(n_samples), input, output, tubedelay);
}

void GxTubeDelay::connect_all_mono_ports(uint32_t port, void* data)
{
  connect_mono(port,data);
  tubedelay->connect_ports(port,data, tubedelay);
}


void GxTubeDelay::clean()
{
#ifndef __SSE__
  wn->delete_instance(wn);;
#endif
  tubedelay->delete_instance(tubedelay);
}


///////////////////////////// LV2 defines //////////////////////////////

static LV2_Handle
instantiate(const LV2_Descriptor*     descriptor,
            double                    rate,
            const char*               bundle_path,
            const LV2_Feature* const* features)
{

  GxTubeDelay *self = new GxTubeDelay();
  if (!self)
    {
      return NULL;
    }

  self->init_dsp_mono((uint32_t)rate);

  return (LV2_Handle)self;
}

static void
connect_port(LV2_Handle instance,
             uint32_t   port,
             void*      data)
{
  GxTubeDelay* self = (GxTubeDelay*)instance;
  self->connect_all_mono_ports(port, data);
}

static void
activate(LV2_Handle instance)
{
 
}

static void
run(LV2_Handle instance, uint32_t n_samples)
{
  GxTubeDelay* self = (GxTubeDelay*)instance;
  self->run_dsp_mono(n_samples);
}

static void
cleanup(LV2_Handle instance)
{
  GxTubeDelay* self = (GxTubeDelay*)instance;
  self->clean();
  delete self;
}

//////////////////////////////////////////////////////////////////

static const LV2_Descriptor descriptor =
{
  GXPLUGIN_URI "#tubedelay",
  instantiate,
  connect_port,
  activate,
  run,
  NULL,
  cleanup,
  NULL
};

extern "C"
LV2_SYMBOL_EXPORT
const LV2_Descriptor*
lv2_descriptor(uint32_t index)
{
  switch (index)
    {
    case 0:
      return &descriptor;
    default:
      return NULL;
    }
}
